@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2009 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -50,7 +50,7 @@ namespace BibTeX
             m_entryTypeString = entryTypeToString( m_entryType );
     }
 
-    Entry::Entry( Entry *other )
+    Entry::Entry( const Entry *other )
     {
         copyFrom( other );
     }
@@ -68,13 +68,33 @@ namespace BibTeX
         return new Entry( this );
     }
 
-    QString Entry::text()
+    bool Entry::equals( const Entry &other )
     {
-        QString result = "Id: ";
-        result.append( m_id ).append( "\n" );
+        if ( other.id().compare( id() ) != 0 )
+            return false;
 
         for ( EntryFields::iterator it = m_fields.begin(); it != m_fields.end(); it++ )
+        {
+            EntryField *field1 = *it;
+            EntryField *field2 = other.getField( field1->fieldTypeName() );
+
+            if ( field2 == NULL || field1->value() == NULL || field2->value() == NULL || field1->value()->text().compare( field2->value()->text() ) != 0 )
+                return false;
+        }
+
+        return true;
+    }
+
+    QString Entry::text() const
+    {
+        QString result = "Id: ";
+        result.append( m_id ).append( "  (" ).append( entryTypeString() ).append( ")\n" );
+
+        for ( EntryFields::ConstIterator it = m_fields.begin(); it != m_fields.end(); it++ )
+        {
+            result.append(( *it )->fieldTypeName() ).append( ": " );
             result.append(( *it )->value()->text() ).append( "\n" );
+        }
 
         return result;
     }
@@ -96,7 +116,7 @@ namespace BibTeX
         return m_entryType;
     }
 
-    QString Entry::entryTypeString()
+    QString Entry::entryTypeString() const
     {
         return m_entryTypeString;
     }
@@ -111,47 +131,81 @@ namespace BibTeX
         return m_id;
     }
 
-    bool Entry::containsPattern( const QString & pattern, EntryField::FieldType fieldType, bool caseSensitive )
+    bool Entry::containsPattern( const QString & pattern, EntryField::FieldType fieldType, BibTeX::Element::FilterType filterType, bool caseSensitive ) const
     {
-        bool result = fieldType == EntryField::ftUnknown && m_id.contains( pattern, caseSensitive );
+        if ( filterType == ftExact )
+        {
+            /** check for exact match */
+            bool result = fieldType == EntryField::ftUnknown && m_id.contains( pattern, caseSensitive );
 
-        for ( EntryFields::iterator it = m_fields.begin(); !result && it != m_fields.end(); it++ )
-            if ( fieldType == EntryField::ftUnknown || ( *it ) ->fieldType() == fieldType )
-                result |= ( *it ) ->value() ->containsPattern( pattern, caseSensitive );
+            for ( EntryFields::ConstIterator it = m_fields.begin(); !result && it != m_fields.end(); it++ )
+                if ( fieldType == EntryField::ftUnknown || ( *it ) ->fieldType() == fieldType )
+                    result |= ( *it ) ->value() ->containsPattern( pattern, caseSensitive );
 
-        return result;
+            return result;
+        }
+        else
+        {
+            /** for each word in the search pattern ... */
+            QStringList words = QStringList::split( QRegExp( "\\s+" ), pattern );
+            bool *hits = new bool[words.count()];
+            int i = 0;
+            for ( QStringList::Iterator wit = words.begin(); wit != words.end(); ++wit, ++i )
+            {
+                hits[i] = fieldType == EntryField::ftUnknown && m_id.contains( *wit, caseSensitive );
+
+                /** check if word is contained in any field */
+                for ( EntryFields::ConstIterator fit = m_fields.begin(); fit != m_fields.end(); ++fit )
+                    if ( fieldType == EntryField::ftUnknown || ( *fit ) ->fieldType() == fieldType )
+                        hits[i] |= ( *fit ) ->value() ->containsPattern( *wit, caseSensitive );
+            }
+
+            unsigned int hitCount = 0;
+            for ( i = words.count() - 1; i >= 0; --i )
+                if ( hits[i] ) ++hitCount;
+            delete[] hits;
+
+            /** return success depending on filter type and number of hits */
+            return (( filterType == ftAnyWord && hitCount > 0 ) || ( filterType == ftEveryWord && hitCount == words.count() ) );
+        }
     }
 
-    QStringList Entry::urls()
+    QStringList Entry::urls() const
     {
         QStringList result;
-        QString fieldNames[] = {"localfile", "pdf", "ps", "postscript", "doi", "url", "howpublished", "ee", "biburl"};
+        const QString fieldNames[] = {"localfile", "pdf", "ps", "postscript", "doi", "url", "howpublished", "ee", "biburl", "note"};
+        const int fieldNamesCount = sizeof( fieldNames ) / sizeof( fieldNames[0] );
 
-        for ( unsigned int i = 0; i < sizeof( fieldNames ) / sizeof( fieldNames[0] ); i++ )
-        {
-            EntryField * field = getField( fieldNames[ i ] );
-            if (( field && !field->value() ->items.isEmpty() ) )
+        for ( int j = 1; j < 5 ; ++j ) /** there may be variants such as url3 or doi2 */
+            for ( int i = 0; i < fieldNamesCount; i++ )
             {
-                PlainText *plainText = dynamic_cast<PlainText*>( field->value()->items.first() );
-                if ( plainText != NULL )
+                QString fieldName = fieldNames[i];
+                /** field names should be like url, url2, url3, ... */
+                if ( j > 1 ) fieldName.append( QString::number( j ) );
+
+                EntryField * field = getField( fieldName );
+                if (( field && !field->value()->items.isEmpty() ) )
                 {
-                    QString plain = plainText->text();
-                    int urlPos = plain.find( "\\url{", 0, FALSE );
-                    if ( urlPos > -1 )
+                    PlainText *plainText = dynamic_cast<PlainText*>( field->value()->items.first() );
+                    if ( plainText != NULL )
                     {
-                        plain = plain.mid( urlPos + 5 );
-                        urlPos = plain.find( "}", 0, FALSE );
-                        if ( urlPos > 0 )
-                            plain = plain.left( urlPos - 1 );
+                        QString plain = plainText->text();
+                        int urlPos = plain.find( "\\url{", 0, FALSE );
+                        if ( urlPos > -1 )
+                        {
+                            plain = plain.mid( urlPos + 5 );
+                            urlPos = plain.find( "}", 0, FALSE );
+                            if ( urlPos > 0 )
+                                plain = plain.left( urlPos - 1 );
+                        }
+
+                        if ( fieldNames[ i ] == "doi" && !plain.startsWith( "http", FALSE ) )
+                            plain.prepend( "http://dx.doi.org/" );
+
+                        result.append( plain );
                     }
-
-                    if ( fieldNames[ i ] == "doi" && !plain.startsWith( "http", FALSE ) )
-                        plain.prepend( "http://dx.doi.org/" );
-
-                    result.append( plain );
                 }
             }
-        }
 
         return result;
     }
@@ -209,14 +263,14 @@ namespace BibTeX
         return FALSE;
     }
 
-    Entry::EntryFields::iterator Entry::begin()
+    Entry::EntryFields::ConstIterator Entry::begin() const
     {
-        return m_fields.begin();
+        return m_fields.constBegin();
     }
 
-    Entry::EntryFields::iterator Entry::end()
+    Entry::EntryFields::ConstIterator Entry::end() const
     {
-        return m_fields.end();
+        return m_fields.constEnd();
     }
 
     int Entry::getFieldCount() const
@@ -231,17 +285,19 @@ namespace BibTeX
         m_fields.clear();
     }
 
-    void Entry::copyFrom( Entry *other )
+    void Entry::copyFrom( const Entry *other )
     {
+        if ( other == NULL ) return;
+
         m_entryType = other->m_entryType;
         m_entryTypeString = other->m_entryTypeString;
         m_id = other->m_id;
         clearFields();
-        for ( EntryFields::iterator it = other->m_fields.begin(); it != other->m_fields.end(); it++ )
+        for ( EntryFields::ConstIterator it = other->m_fields.begin(); it != other->m_fields.end(); it++ )
             m_fields.append( new EntryField( *it ) );
     }
 
-    void Entry::merge( BibTeX::Entry *other )
+    void Entry::merge( BibTeX::Entry *other, MergeSemantics mergeSemantics )
     {
         for ( EntryFields::iterator it = other->m_fields.begin(); it != other->m_fields.end(); it++ )
         {
@@ -250,12 +306,16 @@ namespace BibTeX
             QString otherFieldTypeName = otherField->fieldTypeName();
             EntryField *thisField = otherFieldType != EntryField::ftUnknown ? getField( otherFieldType ) : getField( otherFieldTypeName );
 
-            if ( thisField != NULL )
+            if ( thisField == NULL )
+            {
+                m_fields.append( otherField );
+            }
+            else if ( otherField->value()->text() == thisField->value()->text() && mergeSemantics == msForceAdding )
             {
                 otherFieldTypeName.prepend( "OPT" );
                 otherField->setFieldType( EntryField::ftUnknown, otherFieldTypeName );
+                m_fields.append( otherField );
             }
-            m_fields.append( otherField );
         }
     }
 
@@ -371,6 +431,7 @@ namespace BibTeX
             case EntryField::ftYear:
                 return Entry::frsRequired;
             case EntryField::ftVolume:
+            case EntryField::ftNumber:
             case EntryField::ftSeries:
             case EntryField::ftAddress:
             case EntryField::ftDoi:
