@@ -28,6 +28,8 @@
 #include <kdialogbase.h>
 
 #include <entry.h>
+#include <entrywidgetsource.h>
+#include <webquery.h>
 
 class QComboBox;
 class QLineEdit;
@@ -37,22 +39,28 @@ class QString;
 class QTimer;
 class QListView;
 
+class KConfig;
+
 namespace KBibTeX
 {
     class EntryWidgetTab;
+    class WebQueryArXiv;
 
     class EntryWidget : public QWidget
     {
         Q_OBJECT
+
+        friend class EntryWidgetDialog;
+
     public:
         ~EntryWidget();
 
         static QDialog::DialogCode execute( BibTeX::Entry *entry, BibTeX::File *bibtexfile, bool isReadOnly, bool isNew, QWidget *parent = 0, const char *name = 0 );
-        void closeEvent( QCloseEvent * e );
 
     protected:
-        EntryWidget( BibTeX::File *bibtexfile, bool isReadOnly, bool isNew, QWidget *parent = 0, const char *name = 0 );
+        EntryWidget( BibTeX::File *bibtexfile, bool isReadOnly, bool isNew, QDialog *parent, const char *name = 0 );
         void showEvent( QShowEvent * );
+        bool queryClose();
 
     private slots:
         void apply();
@@ -67,12 +75,15 @@ namespace KBibTeX
         void updateWarnings();
         void insertIdSuggestion( int id );
         void updateIdSuggestionsMenu();
+        void refreshFromURL();
+        void useExternalEntry( BibTeX::Entry*, bool );
+        void endExternalSearch( WebQuery::Status );
 
     signals:
         void updateTabs( BibTeX::Entry::EntryType entryType, bool enableAll, bool isReadOnly );
 
     private:
-        EntryWidget( BibTeX::Entry *entry, BibTeX::File *bibtexfile, bool isReadOnly, bool isNew, QWidget *parent = 0, const char *name = 0 );
+        EntryWidget( BibTeX::Entry *entry, BibTeX::File *bibtexfile, bool isReadOnly, bool isNew, QDialog *parent, const char *name = 0 );
         BibTeX::Entry *m_originalEntry;
         BibTeX::File *m_bibtexfile;
         bool m_isReadOnly;
@@ -86,12 +97,16 @@ namespace KBibTeX
         QPopupMenu *m_menuIdSuggestions;
         QMap<int, QString> m_idToSuggestion;
         QCheckBox *m_checkBoxEnableAll;
+        QPushButton *m_pushButtonRefetch;
         QListView *m_listViewWarnings;
         QTabWidget *m_tabWidget;
-        KBibTeX::EntryWidgetTab *m_sourcePage;
+        KBibTeX::EntryWidgetSource *m_sourcePage;
         QValueList<KBibTeX::EntryWidgetTab*> m_internalEntryWidgets;
         QWidget *m_lastPage;
         QTimer *m_updateWarningsTimer;
+        QDialog *m_dlgParent;
+        WebQueryArXiv *m_wqa;
+        QString m_oldId;
 
         void setupGUI( QWidget *parent, bool showWarnings = TRUE );
         void addTabWidgets();
@@ -100,6 +115,8 @@ namespace KBibTeX
         void updateGUI();
         void internalApply( BibTeX::Entry *entry );
         void internalReset( BibTeX::Entry *entry );
+        void restoreWindowSize( KConfig *config );
+        void saveWindowSize( KConfig *config ) const;
     };
 
     class EntryWidgetDialog: public KDialogBase
@@ -117,7 +134,19 @@ namespace KBibTeX
     protected:
         void closeEvent( QCloseEvent * e )
         {
-            m_widget->closeEvent( e );
+            if ( m_widget->queryClose() ) e->accept();
+            else e->ignore();
+        }
+
+    protected slots:
+        void slotCancel()
+        {
+            if ( m_widget->queryClose() ) QDialog::done( Cancel );
+        }
+
+        void slotClose()
+        {
+            if ( m_widget->queryClose() ) QDialog::done( Cancel );
         }
 
     private:
