@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2007 by Thomas Fischer                             *
+*   Copyright (C) 2004-2009 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -31,8 +31,8 @@
 namespace KBibTeX
 {
 
-    EntryWidgetMisc::EntryWidgetMisc( BibTeX::Entry *entry, BibTeX::File *bibtexfile, bool isReadOnly, QWidget *parent, const char *name )
-            : EntryWidgetTab( entry, bibtexfile, isReadOnly, parent, name )
+    EntryWidgetMisc::EntryWidgetMisc( BibTeX::File *bibtexfile, bool isReadOnly, QWidget *parent, const char *name )
+            : EntryWidgetTab( bibtexfile, isReadOnly, parent, name )
     {
         setupGUI();
     }
@@ -62,44 +62,44 @@ namespace KBibTeX
         m_fieldLineEditAnnote->setEnabled( enableWidget );
     }
 
-    void EntryWidgetMisc::apply( )
+    void EntryWidgetMisc::apply( BibTeX::Entry *entry )
     {
         BibTeX::Value * value = m_fieldLineEditType->value();
-        setValue( m_entry, BibTeX::EntryField::ftType, value );
+        setValue( entry, BibTeX::EntryField::ftType, value );
         delete value;
 
         value = m_fieldLineEditKey-> value();
-        setValue( m_entry, BibTeX::EntryField::ftKey, value );
+        setValue( entry, BibTeX::EntryField::ftKey, value );
         delete value;
 
         value = m_fieldLineEditNote-> value();
-        setValue( m_entry, BibTeX::EntryField::ftNote, value );
+        setValue( entry, BibTeX::EntryField::ftNote, value );
         delete value;
 
         value = m_fieldLineEditAnnote-> value();
-        setValue( m_entry, BibTeX::EntryField::ftAnnote, value );
+        setValue( entry, BibTeX::EntryField::ftAnnote, value );
         delete value;
 
         value = m_fieldLineEditAbstract-> value();
-        setValue( m_entry, BibTeX::EntryField::ftAbstract, value );
+        setValue( entry, BibTeX::EntryField::ftAbstract, value );
         delete value;
     }
 
-    void EntryWidgetMisc::reset( )
+    void EntryWidgetMisc::reset( BibTeX::Entry *entry )
     {
-        BibTeX::EntryField * field = m_entry->getField( BibTeX::EntryField::ftType );
+        BibTeX::EntryField * field = entry->getField( BibTeX::EntryField::ftType );
         m_fieldLineEditType->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftKey );
+        field = entry->getField( BibTeX::EntryField::ftKey );
         m_fieldLineEditKey->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftNote );
+        field = entry->getField( BibTeX::EntryField::ftNote );
         m_fieldLineEditNote->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftAnnote );
+        field = entry->getField( BibTeX::EntryField::ftAnnote );
         m_fieldLineEditAnnote->setValue( field != NULL ? field->value() : NULL );
 
-        field = m_entry->getField( BibTeX::EntryField::ftAbstract );
+        field = entry->getField( BibTeX::EntryField::ftAbstract );
         m_fieldLineEditAbstract->setValue( field != NULL ? field->value() : NULL );
     }
 
@@ -135,30 +135,35 @@ namespace KBibTeX
         QLabel *label = new QLabel( QString( "%1:" ).arg( i18n( "Type" ) ), this );
         gridLayout->addWidget( label, 0, 0 );
         m_fieldLineEditType = new KBibTeX::FieldLineEdit( i18n( "Type" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditType" );
+        m_fieldLineEditType->setFieldType( BibTeX::EntryField::ftType );
         label->setBuddy( m_fieldLineEditType );
         gridLayout->addWidget( m_fieldLineEditType, 0, 1 );
 
         label = new QLabel( QString( "%1:" ).arg( i18n( "Key" ) ), this );
         gridLayout->addWidget( label, 0, 3 );
         m_fieldLineEditKey = new KBibTeX::FieldLineEdit( i18n( "Key" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditKey" );
+        m_fieldLineEditKey->setFieldType( BibTeX::EntryField::ftKey );
         label->setBuddy( m_fieldLineEditKey );
         gridLayout->addWidget( m_fieldLineEditKey, 0, 4 );
 
         label = new QLabel( QString( "%1:" ).arg( i18n( "Note" ) ), this );
         gridLayout->addWidget( label, 1, 0 );
         m_fieldLineEditNote = new KBibTeX::FieldLineEdit( i18n( "Note" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditNote" );
+        m_fieldLineEditNote->setFieldType( BibTeX::EntryField::ftNote );
         label->setBuddy( m_fieldLineEditNote );
         gridLayout->addMultiCellWidget( m_fieldLineEditNote, 1, 1, 1, 4 );
 
         label = new QLabel( QString( "%1:" ).arg( i18n( "Annote" ) ), this );
         gridLayout->addWidget( label, 2, 0 );
         m_fieldLineEditAnnote = new KBibTeX::FieldLineEdit( i18n( "Annote" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditAnnote" );
+        m_fieldLineEditAnnote->setFieldType( BibTeX::EntryField::ftAnnote );
         label->setBuddy( m_fieldLineEditAnnote );
         gridLayout->addMultiCellWidget( m_fieldLineEditAnnote, 2, 2, 1, 4 );
 
         label = new QLabel( QString( "%1:" ).arg( i18n( "Abstract" ) ), this );
         gridLayout->addWidget( label, 3, 0 );
         m_fieldLineEditAbstract = new KBibTeX::FieldLineEdit( i18n( "Abstract" ), KBibTeX::FieldLineEdit::itMultiLine, m_isReadOnly, this, "m_fieldLineEditAbstract" );
+        m_fieldLineEditAbstract->setFieldType( BibTeX::EntryField::ftAbstract );
         label->setBuddy( m_fieldLineEditAbstract );
         gridLayout->addMultiCellWidget( m_fieldLineEditAbstract, 3, 4, 1, 4 );
     }
