@@ -192,7 +192,7 @@ namespace BibTeX
         for ( Entry::EntryFields::ConstIterator it = entry->begin(); it != entry->end(); ++it )
         {
             EntryField *field = *it;
-            QString text = valueToString( field->value(), field->fieldType() );
+            QString text = valueToString( field->value(), field->fieldType(), field->fieldTypeName() );
             if ( m_protectCasing && dynamic_cast<BibTeX::PlainText*>( field->value()->items.first() ) != NULL && ( field->fieldType() == EntryField::ftTitle || field->fieldType() == EntryField::ftBookTitle || field->fieldType() == EntryField::ftSeries ) )
                 addProtectiveCasing( text );
             writeString( device, QString( ",\n\t%1 = %2" ).arg( field->fieldTypeName() ).arg( text ) );
@@ -220,7 +220,6 @@ namespace BibTeX
 
             if ( m_encoding == "latex" )
                 text = EncoderLaTeX::currentEncoderLaTeX() ->encode( text );
-            fixLaTeXChars( text, false );
 
             QStringList commentLines = QStringList::split( '\n', text );
             for ( QStringList::Iterator it = commentLines.begin(); it != commentLines.end(); it++ )
@@ -235,7 +234,6 @@ namespace BibTeX
 
             if ( m_encoding == "latex" )
                 text = EncoderLaTeX::currentEncoderLaTeX() ->encode( text );
-            fixLaTeXChars( text, false );
 
             writeString( device, QString( "@%1{%2}\n\n" ).arg( applyKeywordCasing( "Comment" ) ).arg( text ) );
         }
@@ -251,30 +249,27 @@ namespace BibTeX
 
     bool FileExporterBibTeX::writeString( QIODevice &device, const QString& text )
     {
-        const int encodeMaxTries = 1024;
-        QString workingText = text;
         size_t utf8datasize = 1;
-        int i;
-        for ( i = 0; i < encodeMaxTries && utf8datasize != 0; ++i )
+        QCString utf8 = text.utf8();
+        char *utf8data = utf8.data();
+        utf8datasize = utf8.length();
+        char *outputdata = m_iconvBuffer;
+        size_t outputdatasize = m_iconvBufferSize;
+
+        size_t result = iconv( m_iconvHandle, &utf8data, &utf8datasize, &outputdata, &outputdatasize );
+        if ( result != 0 )
         {
-            QCString utf8 = workingText.utf8();
-            char *utf8data = utf8.data();
-            utf8datasize = utf8.length();
-            char *outputdata = m_iconvBuffer;
-            size_t outputdatasize = m_iconvBufferSize;
-            size_t result = iconv( m_iconvHandle, &utf8data, &utf8datasize, &outputdata, &outputdatasize );
-
-            if ( device.writeBlock( m_iconvBuffer, m_iconvBufferSize - outputdatasize ) != ( int )( m_iconvBufferSize - outputdatasize ) )
-                return false;
-
-            if ( result != 0 )
-            {
-                workingText = QString::fromUtf8( utf8data, utf8datasize );
-                workingText = EncoderLaTeX::currentEncoderLaTeX()->encode( workingText, workingText[0] );
-            }
+            qWarning( "Cannot convert string using iconv" );
+            return false;
         }
 
-        return i < encodeMaxTries;
+        if ( device.writeBlock( m_iconvBuffer, m_iconvBufferSize - outputdatasize ) != ( int )( m_iconvBufferSize - outputdatasize ) )
+        {
+            qWarning( "Cannot write string to device" );
+            return false;
+        }
+
+        return true;
     }
 
     void FileExporterBibTeX::setStringDelimiter( const QChar& stringOpenDelimiter, const QChar& stringCloseDelimiter )
@@ -298,7 +293,7 @@ namespace BibTeX
         m_protectCasing = protectCasing;
     }
 
-    QString FileExporterBibTeX::valueToString( const Value *value, const EntryField::FieldType fieldType )
+    QString FileExporterBibTeX::valueToString( const Value *value, const EntryField::FieldType fieldType, const QString &fieldTypeName )
     {
         if ( value == NULL )
             return "";
@@ -372,19 +367,23 @@ namespace BibTeX
                               * Thus we split the von-Parts from the surname (= everything after the first upcase char).
                               * FIXME: Make the personContainer aware of von-Parts and jr-Parts, instead.
                               */
-                            QStringList list = QStringList::split(" ", v);
+                            QStringList list = QStringList::split( " ", v );
                             QString von;
-                            for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
+                            for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
+                            {
                                 QString str = *it;
-                                if (str != "others" && str[0].category() == QChar::Letter_Lowercase) {
+                                if ( str != "others" && str[0].category() == QChar::Letter_Lowercase )
+                                {
                                     von += *it;
                                     von += " ";
-                                } else
+                                }
+                                else
                                     break;
                             }
-                            if (!von.isEmpty()) {
+                            if ( !von.isEmpty() )
+                            {
                                 text.append( von );
-                                v = v.right(v.length() - von.length());
+                                v = v.right( v.length() - von.length() );
                             }
                             bool requiresQuoting = requiresPersonQuoting( v, TRUE );
                             if ( requiresQuoting ) text.append( "{" );
@@ -397,7 +396,8 @@ namespace BibTeX
                 if ( m_encoding == "latex" )
                     text = encoder->encodeSpecialized( text, fieldType );
 
-                fixLaTeXChars( text, fieldType == EntryField::ftURL );
+                if ( fieldType == EntryField::ftURL || fieldType == EntryField::ftDoi || ( fieldType == EntryField::ftUnknown && fieldTypeName.lower() == "slaccitation" ) )
+                    removeBackslashQuoting( text );
 
                 /** if the text to save contains a quote char ("),
                   * force string delimiters to be curly brackets,
@@ -419,48 +419,9 @@ namespace BibTeX
         return result;
     }
 
-    void FileExporterBibTeX::fixLaTeXChars( QString &text, bool verbatim )
+    void FileExporterBibTeX::removeBackslashQuoting( QString &text )
     {
-        if ( verbatim )
-        {
-            /** for verbatim cases, remove all backslash quotings */
-            text.replace( "\\&", "&" ).replace( "\\#", "#" ).replace( "\\_", "_" ).replace( "\\%", "%" );
-        }
-        else
-        {
-            /** in the first phase, add backslashes before special characters to be protected */
-            /** Use regular expressions to replace e.g. "e&" by "e\&" */
-            const char protect[] = {'&', '#', '_', '%'};
-            const QString prefix = "([^\\\\])";
-            for ( int i = sizeof( protect ) / sizeof( protect[0] ) - 1; i >= 0; --i )
-            {
-                QRegExp rx = QRegExp( prefix + protect[i] );
-                int pos = -1;
-                while (( pos = rx.search( text, pos + 1 ) ) >= 0 )
-                {
-                    text.replace( rx.cap( 1 ) + protect[i], rx.cap( 1 ) + "\\" + protect[i] );
-                }
-                /** special case at start of line */
-                text.replace( QRegExp( QString( "^" ) + protect[i] ), QString( "\\" ) + protect[i] );
-            }
-
-            /** handle special verbatim environment \url{..} by removing backslashes added just above */
-            int pos = -1;
-            while (( pos = text.find( "\\url{", pos + 1 ) ) >= 0 )
-            {
-                int p2 = pos + 5;
-                int counter = 1;
-                while ( counter > 0 )
-                {
-                    if ( text[p2] == '{' )++counter;
-                    else if ( text[p2] == '}' )--counter;
-                    ++p2;
-                }
-                QString text2 = text.mid( pos + 5, p2 - pos - 6 );
-                fixLaTeXChars( text2, true );
-                text = text.left( pos + 5 ) + text2 + text.mid( p2 - 1 );
-            }
-        }
+        text.replace( "\\&", "&" ).replace( "\\#", "#" ).replace( "\\_", "_" ).replace( "\\%", "%" );
     }
 
     QString FileExporterBibTeX::applyKeywordCasing( const QString &keyword )
