@@ -110,12 +110,14 @@ namespace BibTeX
             QTextStream ts( &latexFile );
             ts.setEncoding( QTextStream::UnicodeUTF8 );
             ts << "\\documentclass{article}\n";
-            if ( kpsewhich( "t2aenc.def" ) && kpsewhich( "t2aenc.dfu" ) &&  kpsewhich( "t1enc.dfu" ) )
-                ts << "\\usepackage[T1,T2A]{fontenc}\n";
+            if ( kpsewhich( "t1enc.dfu" ) )
+                ts << "\\usepackage[T1]{fontenc}\n";
             if ( kpsewhich( "babel.sty" ) )
                 ts << "\\usepackage[" << m_latexLanguage << "]{babel}\n";
             if ( kpsewhich( "url.sty" ) )
                 ts << "\\usepackage{url}\n";
+            if ( m_latexBibStyle.startsWith( "apacite" ) && kpsewhich( "apacite.sty" ) )
+                ts << "\\usepackage[bibnewpage]{apacite}\n";
             ts << "\\bibliographystyle{" << m_latexBibStyle << "}\n";
             ts << "\\begin{document}\n";
             ts << "\\nocite{*}\n";
