@@ -220,11 +220,16 @@ bool KBibTeXPart::saveFile()
         KTempFile tempBibFile = KTempFile( locateLocal( "tmp", "bibsave" ) );
         tempBibFile.setAutoDelete( TRUE );
         bool success =  m_documentWidget->save( tempBibFile.file() );
+        tempBibFile.close();
         if ( success )
         {
-            tempBibFile.close();
+            QString usedFileName = m_file;
+            QFileInfo fi( usedFileName );
+            if ( !fi.readLink().isNull() && KMessageBox::questionYesNo( widget(), QString( i18n( "The selected filename \"%1\" is a symbolic link pointing to \"%2\".\nReplace the link with a new file or overwrite the existing file the link points to?" ) ).arg( usedFileName ).arg(KBibTeX::Settings::resolveLink( m_file, fi.readLink() ) ), i18n( "Symbolic Link" ), KGuiItem( i18n( "Replace link" ) ), KGuiItem( i18n( "Overwrite file the link points to" ) ) ) == KMessageBox::No )
+                usedFileName = KBibTeX::Settings::resolveLink( m_file, fi.readLink() );
+
             m_documentWidget->dirWatch().stopScan();
-            success = KIO::NetAccess::file_copy( KURL( tempBibFile.name() ), KURL( m_file ), -1, true, false, widget() );
+            success = KIO::NetAccess::file_copy( KURL( tempBibFile.name() ), KURL( usedFileName ), -1, true, false, widget() );
             m_documentWidget->dirWatch().startScan();
         }
 
@@ -248,7 +253,6 @@ bool KBibTeXPart::saveAs()
 
     if ( saveURL.isValid() && !saveURL.isEmpty() )
     {
-
         if ( KIO::NetAccess::exists( saveURL, FALSE, widget() ) && KMessageBox::warningContinueCancel( widget(), i18n( "A file named '%1' already exists. Are you sure you want to overwrite it?" ).arg( saveURL.filename() ), QString::null, i18n( "Overwrite" ) ) != KMessageBox::Continue )
             return result;
 
@@ -735,7 +739,7 @@ KInstance* KBibTeXPartFactory::instance()
 {
     if ( !s_instance )
     {
-        s_about = new KAboutData( "kbibtexpart", I18N_NOOP( "KBibTeXPart" ), "0.2.2" );
+        s_about = new KAboutData( "kbibtexpart", I18N_NOOP( "KBibTeXPart" ), "0.2.3.90" );
         s_about->addAuthor( "Thomas Fischer", 0, "fischer@unix-ag.uni-kl.de" );
         s_instance = new KInstance( s_about );
     }
